@@ -206,4 +206,7 @@ darray_next_alloc(unsigned alloc, unsigned need, unsigned itemSize)
          (idx) < (arr).size; \
          (idx)++, (val)++)
 
+#define darray_foreach_reverse(i, arr) \
+    for ((i) = &(arr).item[(arr).size - 1]; (arr).size > 0 && (i) >= &(arr).item[0]; (i)--)
+
 #endif /* CCAN_DARRAY_H */
