@@ -399,6 +399,13 @@ matcher_include(struct matcher *m, struct scanner *parent_scanner,
                     return;
                 }
             }
+            else if (chr(&s, 'E')) {
+                const char *default_root = xkb_context_include_path_get_extra_path(m->ctx);
+                if (!buf_appends(&s, default_root) || !buf_appends(&s, "/rules")) {
+                    scanner_err(&s, "include path after expanding %%E is too long");
+                    return;
+                }
+            }
             else {
                 scanner_err(&s, "unknown %% format (%c) in include statement", peek(&s));
                 return;
@@ -1104,8 +1111,9 @@ xkb_components_from_rules(struct xkb_context *ctx,
     char *path = NULL;
     struct matcher *matcher = NULL;
     struct matched_sval *mval;
+    unsigned int offset = 0;
 
-    file = FindFileInXkbPath(ctx, rmlvo->rules, FILE_TYPE_RULES, &path);
+    file = FindFileInXkbPath(ctx, rmlvo->rules, FILE_TYPE_RULES, &path, &offset);
     if (!file)
         goto err_out;
 
